﻿using ELTE.BoardGame.Persistence;
using ELTE.BoardGame.Model;
using System;
using System.Drawing;
using System.Windows.Forms;

namespace ELTE.BoardGame.View
{
    /// <summary>
    /// Táblajáték ablak típusa.
    /// </summary>
    public partial class DrawingForm : Form
    {
        #region Private fields

        private BoardGameModel model; // játékmodell

        #endregion

        #region Constructors

        /// <summary>
        /// Tic-Tac-Toe ablak létrehozása.
        /// </summary>
        /// <param name="model">A Tic-Tac-Toe játék.</param>
        public DrawingForm(BoardGameModel model)
        {
            if (model == null)
                throw new ArgumentNullException("model", "Nincs megadva játék");
            
            this.InitializeComponent();

            // eseménykezelők társítása
            this.model = model;
            this.model.FieldChanged += new EventHandler<FieldChangedEventArgs>(Model_FieldChanged);
            this.model.GameOver += new EventHandler(Model_GameOver);
            this.model.GameWon += new EventHandler<GameWonEventArgs>(Model_GameWon);

            // új játék indítása
            this.model.NewGame();

            KeyPreview = true; // elfogjuk a billentyűzeteseményeket
            KeyDown += new KeyEventHandler(TicTacToeForm_KeyDown);
        }

        #endregion

        #region Model event handlers

        /// <summary>
        /// Játék megnyerésének eseménykezelése.
        /// </summary>
        private void Model_GameWon(object sender, GameWonEventArgs e)
        {
            switch (e.Player)
            {
                case PlayerData.PlayerO:
                    MessageBox.Show("A kör játékos győzött!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    break;
                case PlayerData.PlayerX:
                    MessageBox.Show("A kereszt játékos győzött!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    break;
            }
            this.model.NewGame();

            this.panel.Refresh();
        }

        /// <summary>
        /// Játék végének eseménykezelése.
        /// </summary>
        private void Model_GameOver(object sender, EventArgs e)
        {
            MessageBox.Show("Döntetlen játék!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            this.model.NewGame();

            this.panel.Refresh();
        }

        /// <summary>
        /// Modell mezőváltozásának eseménykezelése.
        /// </summary>
        private void Model_FieldChanged(object sender, FieldChangedEventArgs e)
        {
            this.panel.Refresh();
        }

        #endregion

        #region Panel event handlers

        /// <summary>
        /// Panel kirajzolásának eseménykezelése.
        /// </summary>
        private void Panel_Paint(object sender, PaintEventArgs e)
        {
            Bitmap bitmap = new Bitmap(this.panel.Width, this.panel.Height); // kép a hatékony kirajzoláshoz

            Graphics graphics = Graphics.FromImage(bitmap);
            graphics.Clear(Color.White); // háttér fehérré festése

            // játéktábla rácsai
            Int32 fieldWidth = this.panel.Width / this.model.BoardWidth;
            Int32 fieldHeight = this.panel.Height / this.model.BoardHeight;

            for (Int32 i = 0; i < this.model.BoardWidth; i++)
                graphics.DrawLine(Pens.Black, 0, i * fieldHeight, this.panel.Width, i * fieldHeight);

            for (Int32 j = 0; j < this.model.BoardHeight; j++)
                graphics.DrawLine(Pens.Black, j * fieldWidth, 0, j * fieldWidth, this.panel.Height);

            // a mezőtartalmak
            for (Int32 i = 0; i < this.model.BoardWidth; i++)
                for (Int32 j = 0; j < this.model.BoardHeight; j++)
                {
                    switch (this.model[i, j])
                    { 
                        case PlayerData.PlayerO:
                            graphics.FillEllipse(Brushes.Yellow, i * fieldWidth + fieldWidth / 10, j * fieldHeight + fieldHeight / 10, 8 * fieldWidth / 10, 8 * fieldHeight / 10);
                            break;
                        case PlayerData.PlayerX:
                            graphics.DrawLine(new Pen(Color.Orange, this.panel.Width / 50), i * fieldWidth + fieldWidth / 10, j * fieldHeight + fieldHeight / 10, i * fieldWidth + 9 * fieldWidth / 10, j * fieldHeight + 9 * fieldHeight / 10);
                            graphics.DrawLine(new Pen(Color.Orange, this.panel.Width / 50), i * fieldWidth + 9 * fieldWidth / 10, j * fieldHeight + fieldHeight / 10, i * fieldWidth + fieldWidth / 10, j * fieldHeight + 9 * fieldHeight / 10);
                            break;
                    }
                }

            e.Graphics.DrawImage(bitmap, 0, 0);
        }

        /// <summary>
        /// Panel egérlenyomásának eseménykezelése.
        /// </summary>
        private void Panel_MouseDown(object sender, MouseEventArgs e)
        {
            // megállapítjuk, melyik mezőn van az egér
            Int32 x = e.X * this.model.BoardWidth / this.panel.Width;
            Int32 y = e.Y * this.model.BoardHeight / this.panel.Height;

            try
            {
                this.model.StepGame(x, y); // lépünk a játékban
            }
            catch { }
        }

        #endregion

        #region Form event handlers
        
        /// <summary>
        /// Ablak átmérezetésének eseménykezelése.
        /// </summary>
        private void TicTacToeForm_Resize(object sender, EventArgs e)
        {
            this.panel.Refresh();
        }

        /// <summary>
        /// Billentyű lenyomásának eseménykezelője.
        /// </summary>
        private void TicTacToeForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (!e.Control) // ha nincs lenyomva a Control, nem csinálunk semmit
                return;

            switch (e.KeyCode)
            { 
                case Keys.N: // új játék indítása (Ctrl+N)
                    this.model.NewGame();
                    break;
                case Keys.L: // betöltés (Ctrl+L)
                    if (openFileDialog.ShowDialog() == DialogResult.OK)
                    {
                        try
                        {
                            this.model.LoadGame(openFileDialog.FileName);
                        }
                        catch (DataException)
                        {
                            MessageBox.Show("Hiba keletkezett a betöltés során.", "Tic-Tac-Toe", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    break;
                case Keys.S: // mentés (Ctrl+S)
                    if (saveFileDialog.ShowDialog() == DialogResult.OK)
                    {
                        try
                        {
                            this.model.SaveGame(saveFileDialog.FileName);
                        }
                        catch (DataException)
                        {
                            MessageBox.Show("Hiba keletkezett a mentés során.", "Tic-Tac-Toe", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    break;
            }
            this.panel.Refresh();
        }

        #endregion
    }
}
